var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
define(["require", "exports", "@uifabricshared/foundation-composable", "@uifabricshared/foundation-settings", "@uifabricshared/theming-ramp", "@uifabricshared/foundation-tokens", "@uifabricshared/themed-settings"], function (require, exports, foundation_composable_1, foundation_settings_1, theming_ramp_1, foundation_tokens_1, themed_settings_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Get the cache for the given component from the theme, creating it if necessary
     *
     * @param component - component to get the cache for, the component object itself will store the unique symbol for its lookups
     * @param theme - theme where the cache will be stored
     */
    function _getComponentCache(component, theme) {
        var cacheKey = component.tokenCacheKey;
        theme[cacheKey] = theme[cacheKey] || {};
        return theme[cacheKey];
    }
    /**
     * Create a new render data object with props and theme.  State is an empty object unless this routine is
     * overridden.  At that point a component can store what they wish in state.
     *
     * @param props - input props for the component, these will be shallow copied to allow mutating the object
     * @param theme - theme to set into the render data
     */
    function standardPrepareRenderData(props, theme) {
        return {
            props: __assign({}, props),
            theme: theme,
            state: {}
        };
    }
    exports.standardPrepareRenderData = standardPrepareRenderData;
    function standardThemeQueryInputs(name, renderData) {
        return { name: name, overrides: renderData.props };
    }
    exports.standardThemeQueryInputs = standardThemeQueryInputs;
    function getSettingsFromTheme(theme, name) {
        return theming_ramp_1.getSettings(theme, name).settings;
    }
    function _processSettings(component, data) {
        var overrides = component.themeQueryInputs(component.displayName, data).overrides;
        var cache = _getComponentCache(component, data.theme);
        var _a = themed_settings_1.getThemedSettings(component.settings, data.theme, cache, component.displayName, overrides, getSettingsFromTheme), settings = _a.settings, key = _a.key;
        data.slotProps = settings;
        data.settingsKey = key;
        return data;
    }
    exports._processSettings = _processSettings;
    /**
     * Run the applicable token processors, caching the result so that other control instances with the same props combinations
     * can re-use the results
     *
     * @param component - component options that holds the token processors
     * @param renderData - render data to update with the results
     */
    function _processTokens(component, renderData) {
        if (component.resolvedTokens) {
            var props = renderData.props, theme = renderData.theme, slotProps = renderData.slotProps, settingsKey = renderData.settingsKey;
            var cache = _getComponentCache(component, theme);
            renderData.slotProps = foundation_tokens_1.processTokens(props, theme, slotProps, component.resolvedTokens, settingsKey, cache);
        }
        return renderData;
    }
    /**
     * This routine will merge props into slotProps.root, then finalize the result ensuring any new style keys can be authored
     * against the theme names
     *
     * @param renderData - data being prepared for render.
     */
    function standardFinalizer(renderData) {
        renderData.slotProps = foundation_settings_1.mergeSettings(renderData.slotProps, { root: renderData.props });
        return renderData;
    }
    exports.standardFinalizer = standardFinalizer;
    function standardUsePrepareState(renderData) {
        return renderData;
    }
    exports.standardUsePrepareState = standardUsePrepareState;
    /**
     * Process the input props and get all the properties ready to render
     *
     * @param component - component to process the props for
     * @param componentProps - input props passed in
     * @param theme - active theme for this component
     */
    function useProcessComponent(component, userProps, theme) {
        // set up the initial render data and call any hooks for the component
        var renderData = standardPrepareRenderData(userProps, theme);
        renderData = component.usePrepareState(renderData);
        // process the base settings from the theme and any component customizations
        renderData = _processSettings(component, renderData);
        // process tokens if any are specified
        renderData = _processTokens(component, renderData);
        // finally run any finalizers on the props
        renderData = component.finalizer ? component.finalizer(renderData) : standardFinalizer(renderData);
        return renderData;
    }
    exports.useProcessComponent = useProcessComponent;
    /**
     * Either call the view function if one exists, or just do a simple renderSlot call on the root slot
     *
     * @param component - component options to use for rendering
     * @param result - the resolved slot data for this component.  This will include the IRenderData
     */
    function renderComponent(component, result) {
        var children = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            children[_i - 2] = arguments[_i];
        }
        return component.view
            ? component.view.apply(component, [result].concat(children)) : (result.slots && result.slots.root && foundation_composable_1.renderSlot.apply(void 0, [result.slots.root].concat(children))) || null;
    }
    exports.renderComponent = renderComponent;
    /**
     * Take a component options object and create an IComposable wrapper for it
     *
     * @param component - component options for this component
     */
    function wrapComponent(component) {
        return {
            useProcessProps: function (props, theme) {
                return useProcessComponent(component, props, theme);
            },
            render: function (result) {
                var children = [];
                for (var _i = 1; _i < arguments.length; _i++) {
                    children[_i - 1] = arguments[_i];
                }
                return renderComponent.apply(void 0, [component, result].concat(children));
            },
            slots: component.slots ? foundation_composable_1.wrapSlots(component.slots) : undefined
        };
    }
    exports.wrapComponent = wrapComponent;
});
//# sourceMappingURL=Component.js.map