"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = __importStar(require("react"));
var foundation_composable_1 = require("@uifabricshared/foundation-composable");
var Component_1 = require("./Component");
var theming_react_native_1 = require("@uifabricshared/theming-react-native");
var foundation_tokens_1 = require("@uifabricshared/foundation-tokens");
/* tslint:disable-next-line no-any */
function _getOptions(obj) {
    return (obj && typeof obj !== 'object' && obj.__options) || undefined;
}
function _getComponentOptions(inputComponent, base) {
    var baseComposable = _getOptions(base);
    var baseRoot = (!baseComposable && base) || undefined;
    if (baseComposable || baseRoot) {
        // append custom settings to any pre-existing ones
        var parent_1 = baseComposable || {};
        var slots = __assign({}, parent_1.slots, inputComponent.slots);
        var settings = [].concat(parent_1.settings, inputComponent.settings).filter(function (v) { return v; });
        if (baseRoot) {
            slots.root = baseRoot;
        }
        return __assign({}, baseComposable, inputComponent, { slots: slots,
            settings: settings });
    }
    return inputComponent;
}
function _hasToken(slots, target, key) {
    var slot = slots[target];
    var options = _getOptions(slot);
    return options && options.resolvedTokens && options.resolvedTokens.tokenKeys.hasOwnProperty(key);
}
function _setupComponentOptions(options) {
    // ensure functions and the symbol are set up correctly
    options.tokenCacheKey = Symbol(options.displayName);
    options.usePrepareState = options.usePrepareState || Component_1.standardUsePrepareState;
    options.themeQueryInputs = options.themeQueryInputs || Component_1.standardThemeQueryInputs;
    // process the tokens and get them ready to render
    var slots = options.slots;
    var hasTokens = slots
        ? function (target, key) {
            return _hasToken(slots, target, key);
        }
        : undefined;
    options.resolvedTokens = foundation_tokens_1.buildComponentTokens(options.slots, hasTokens);
}
/**
 * Assembles a higher order component based on the following: styles, theme, view, and slots.
 * Imposes a separation of concern and centralizes styling processing to increase ease of use and robustness
 * in how components use and apply styling and theming.
 *
 * @param component - component definition for the component to be created.  See IComponent for more details.
 */
function compose(inputComponent, base) {
    var options = _getComponentOptions(inputComponent, base);
    _setupComponentOptions(options);
    var composable = Component_1.wrapComponent(options);
    var Component = function (userProps) {
        // get the theme value from the context (or the default theme if it is not set)
        var theme = React.useContext(theming_react_native_1.ThemeContext) || theming_react_native_1.getTheme();
        // perform prop resolution as specified by the composable pattern
        var result = foundation_composable_1.useProcessComposableTree(composable, userProps, theme);
        // now use the slot renderer to call the view function
        return foundation_composable_1.renderSlot(result);
    };
    // set the displayName and merge in statics
    Component.displayName = options.displayName;
    Component.__options = options;
    Component.__composable = composable;
    Object.assign(Component, options.statics);
    // set up the customize handler
    Component.customize = function () {
        var settings = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            settings[_i] = arguments[_i];
        }
        return compose({
            displayName: options.displayName + '+',
            settings: settings
        }, Component);
    };
    return Component;
}
exports.compose = compose;
//# sourceMappingURL=compose.js.map