/// <reference types="react" />
import { IComponent, IRenderData, IComponentOptions } from './Component.types';
import { INativeTheme } from '@uifabricshared/theming-react-native';
import { IProcessResult, IResolvedSlot, IComposable, IGenericProps } from '@uifabricshared/foundation-composable';
/**
 * Create a new render data object with props and theme.  State is an empty object unless this routine is
 * overridden.  At that point a component can store what they wish in state.
 *
 * @param props - input props for the component, these will be shallow copied to allow mutating the object
 * @param theme - theme to set into the render data
 */
export declare function standardPrepareRenderData(props: IGenericProps, theme: INativeTheme): IRenderData;
export declare function standardThemeQueryInputs(name: string, renderData: IRenderData): {
    name: string;
    overrides?: object;
};
export declare function _processSettings<TComponent extends IComponent>(component: TComponent, data: IRenderData): IRenderData;
/**
 * This routine will merge props into slotProps.root, then finalize the result ensuring any new style keys can be authored
 * against the theme names
 *
 * @param renderData - data being prepared for render.
 */
export declare function standardFinalizer(renderData: IRenderData): IRenderData;
export declare function standardUsePrepareState(renderData: IRenderData): IRenderData;
/**
 * Process the input props and get all the properties ready to render
 *
 * @param component - component to process the props for
 * @param componentProps - input props passed in
 * @param theme - active theme for this component
 */
export declare function useProcessComponent<TOptions extends IComponentOptions<IComponent>>(component: TOptions, userProps: IGenericProps, theme: object): IProcessResult;
/**
 * Either call the view function if one exists, or just do a simple renderSlot call on the root slot
 *
 * @param component - component options to use for rendering
 * @param result - the resolved slot data for this component.  This will include the IRenderData
 */
export declare function renderComponent<TComponent extends IComponent>(component: TComponent, result: IResolvedSlot, ...children: React.ReactNode[]): JSX.Element | null;
/**
 * Take a component options object and create an IComposable wrapper for it
 *
 * @param component - component options for this component
 */
export declare function wrapComponent<TComponent extends IComponent>(component: TComponent): IComposable;
